function options = hiosdoptions(x)
%% HiOSD options setting
% x       initial value of x, column vector n*1
% V       initial value of V; n*k
% innpfunc can compute multi - one column inner product as a column vector
% k       index of the sddle point
% dt      initial step size 
%         if dt = 0, then the iteration of x is stopped
% l       initial dimer length
% minl    minimal dimer length
% epsf    iteration stops if |F(x)| is smaller than this
% maxiter max iteration step number
% betat   max beta
% betau   min beta
% gamamax max gamma
% gamamin min gamma
% outputX store iteration message every outputX steps
% outputp print iteration message every outputp steps
% outputd draw iteration message every outputp steps
% draws   draw sentence
% tau     max |x-xp| to control step size
% precd   preconditioner for LOBPCG method
% adding  additional sentence every iteration

options=struct('k',1,'dt',0.1,'l',1e-6,'minl',1e-6,...
    'epsf',1e-10,'maxiter',1e3,...
    'betat',max(x,1),'betau',min(1,1/x),'gammamax',max(x,1),'gammamin',min(1,1/x),...
    'outputX',1, 'outputp',1, 'outputd', 1, 'draws', '',...
    'tau',0.5,'precd',@(p)p);

if x==Inf
    options.betat=Inf;
    options.betau=0.01;
    options.gammammax=Inf;
    options.gammammin=1;
end
end
